function [g,err]=edcfit(x,q,K,visu,g0,a,crit);

% Training of evidential distance-based classifier
% [g,err]=edcfit(x,q,K,visu,a);
% Input :
% x = training set, size (n,d), where n=number of examples, d=input dimension
% q = class labels expressed as commonalities, size (n,2^c), where c=number of classes
%     The order of belief masses is the same as assume in Philippe Smets FMT toolbox
% K = number of neighbours
% visu = 1 for visualisation during optimisation (0 otherwise)
% g0=initial value of g (obtained from edcinit)
% a = fixed scalar parameter, 0 < a <= 1 (default=0.99)
% crit = optimization criterion. For each test pattern,
%     crit=1 -> err=1-sum_{omega in Omega} phat(omega)*q({omega})
%               where phat= output pignistic probability
%               q = learning commonality function (label)
%     crit=2 -> err=1-sum_{omega in Omega} phat(omega)*p(omega) 
%               where phat= output pignistic probability
%               p = pignistic probability derived from learning commonality function
%
% Output :
% g = optimized scalar parameter
% err = value of the error criterion 
%
% Uses:
% - the FMT package developed by Philippe Smets 
% - the Matlab Optimization toolbox
%
% References
%
% The basic method is described in
%
% T. Denoeux. A k-nearest neighbor classification rule based on Dempster-Shafer theory. 
%    IEEE Transactions on Systems, Man and Cybernetics, 25(05):804-813, 1995.
%
% The application of the method with possibilistic labels (a special case of the training
% data considered in this program),as well as error criterion 1, are discussed in
%
% T. Denoeux and L. M. Zouhal. Handling possibilistic labels in pattern classification 
%     using evidential reasoning. Fuzzy Sets and Systems, 122(3):47-62, 2001.
%
% Criteria 1 and 2 are presented in
%
% P. Vannoorenbergue and T. Denoeux. Handling uncertain labels in multiclass problems using 
%     belief decision trees IPMU'2002, Annecy, France, July 2002.

if nargin < 6,
   a=0.99;
end;
if length(g0)>1, g0=mean(g0); end;

[n,d]=size(x);
C=size(q,2);
c=round(log(C)./log(2));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% calcul des K ppv dans l'ens. d'app.

ds=zeros(K,n);
Q=zeros(K,n,C);
for i = 1:n,
   dist=sum(((ones(n,1)*x(i,:))-x).^2,2);
   [dss,iss]=sort(dist);
   ds(:,i) = dss(2:K+1);
   Q(:,i,:) = q(iss(2:K+1),:);
end;
Ds=repmat(ds,[1 1 C]);

options=optimset;
if visu,
  options=optimset('Display','iter');
end;
gmin=0.001*g0;
gmax=10*g0;


I=(1:C-1)';
F = zeros(C-1,c);
for i = 1:c 
	F(:,i) = bitget(I,i);  
end 
nF=sum(F,2);
v=(-1).^(nF+1);

U=F'./(ones(c,1)*nF');
U=[zeros(c,1) U];
F0=[zeros(1,c);F];

G=[1];
for i=1:c,
    G=[G zeros(2^(i-1));G G];
end;
G=G';
H=inv(G);

if crit==1,
    i1=find(nF==1);
    qt=q(:,i1+1);
    W=qt*U*H;
else,
    qn=q;
    qn(:,2:C)=qn(:,2:C)./(qn(:,2:C)*v*ones(1,C-1));
    pig=(U*H*qn')';
    W=pig*U*H;
end;  
g=fminbnd('errcrit',gmin,gmax,options,a,Q,Ds,v,W);
err=errcrit(g,a,Q,Ds,v,W);

