function [m,P,s,qh]=edcval(xt,xa,q,g,K,a);

% Evidential distance-based classifier
% [qh,m,P]=edcval(xt,xa,q,g,K,a)
% Input :
% xt = test set, size (nt,d)
% xa = training set, size (na,d)
% q = class labels expressed as commonalities, size (na,2^c)
% g = scale parameter (dimension=1 if version=1, (nt,1) if version=2)
% If version = 2, g should be obtained using edcinit
% If version = 1, g can be obtained either from edcinit (heuristic initial value), or from
% edcfit (optimized value).
% K = number of neighbours
% a = scalar parameter (default=0.99)
% Output :
% m = output belief masses, size (nt,2^c)
% P = output pignistic probabilities, size (nt,c)
% s = predicted class, size (nt,1)
% qh = output commonalities, size (nt,2^c)
%
% Uses the FMT package developed by Philippe Smets 
%
% References
%
% The basic method is described in
%
% T. Denoeux. A k-nearest neighbor classification rule based on Dempster-Shafer theory. 
%    IEEE Transactions on Systems, Man and Cybernetics, 25(05):804-813, 1995.
%
% The application of the method with possibilistic labels (a special case of the training
% data considered in this program),as well as error criterion 1, are discussed in
%
% T. Denoeux and L. M. Zouhal. Handling possibilistic labels in pattern classification 
%     using evidential reasoning. Fuzzy Sets and Systems, 122(3):47-62, 2001.
%
% Criteria 1 and 2 are presented in
%
% P. Vannoorenbergue and T. Denoeux. Handling uncertain labels in multiclass problems using 
%     belief decision trees IPMU'2002, Annecy, France, July 2002.



if nargin< 6,
   a=0.99;
end;

[na,d]=size(xa);
nt=size(xt,1);
C=size(q,2);
c=round(log(C)./log(2));

I=(1:C-1)';
F = zeros(C-1,c);
for i = 1:c 
	F(:,i) = bitget(I,i);  
end 
nF=sum(F,2);
v=(-1).^(nF+1);

if length(g)==1,
   version=1;
else,
   version=2;
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% calcul des K ppv dans l'ens. d'app.

ds=zeros(K,nt);
Q=zeros(K,nt,C);
if version==2, 
   gg=zeros(K,nt);
end;
for i = 1:nt,
   dist=sum(((ones(na,1)*xt(i,:))-xa).^2,2);
   [dss,iss]=sort(dist);
   ds(:,i) = dss(1:K);
   Q(:,i,:) = q(iss(1:K),:);
   if version==2,
      gg(:,i)=g(iss(1:K));
   end;
end;
Ds=repmat(ds,[1 1 C]);

if version==1,
   G=repmat(g,[K nt C]);
else,
   G=repmat(gg,[1 1 C]);
end;
PHI=a*exp(-G.*Ds);
qh=squeeze(prod(PHI.*Q+1-PHI,1));
if nt==1, qh=qh';end;
qh(:,2:C)=qh(:,2:C)./(qh(:,2:C)*v*ones(1,C-1));

m=qh;
P=zeros(nt,c);
for i=1:nt,
   m(i,:)=qtom(qh(i,:));
   P(i,:)=betp(m(i,:));
end;
[temp,s]=max(P,[],2);

